// CellMap.cpp: implementation of the CellMap class.
//
//////////////////////////////////////////////////////////////////////

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

#include "CellMap.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CellMap::CellMap(long dx, long dy, const char * rle)
{
	m_ref = 1;
	m_image = NULL;
	m_width = 0;
	m_height = 0;
	m_dx = dx;
	m_dy = dy;
	readRle(rle);
}


CellMap::~CellMap()
{
	delete [] m_image;
	m_image = NULL;
	m_width = 0;
	m_height = 0;
}

bool CellMap::getCell(long x, long y)
{
	x += m_dx;
	y = m_dy - y;

	if ((x >= 0) && (x < m_width)
		&& (y >= 0) && (y < m_height))
	{
		return m_image[y * m_width + x];
	}
	return false;
}

void CellMap::readRle(const char *rle)
{
	char * p = (char *) rle;
	long pos = 0;

	long num;

	long line;
	long w = 1;
	long h = 1;

	while ((*p != '\0') && (*p != '!'))
	{
		if ((*p >= '0') && (*p <= '9'))
		{
			num = strtol(p, &p, 10);
		}
		else
		{
			num = 1;
		}
		switch (*p)
		{
		case '$':
			h += num;
			if (w < pos)
			{
				w = pos;
			}
			pos = 0;
			break;
		case 'b':
		case 'o':
			pos += num;
			break;
		default:
			fprintf(stderr,"Error reading RLE\n");
			return;
		}
		++p;
	}

	if (w < pos)
	{
		w = pos;
	}

	if (w * h > 10000)
	{
		fprintf(stderr,"RLE too big");
		return;
	}

	m_image = new bool [w * h];
	assert(m_image != NULL);

	m_width = w;
	m_height = h;

	p = (char *) rle;

	line = m_height - 1;
	pos = 0;

	while ((*p != '\0') && (*p != '!'))
	{
		if ((*p >= '0') && (*p <= '9'))
		{
			num = strtol(p, &p, 10);
		}
		else
		{
			num = 1;
		}
		switch (*p)
		{
		case '$':
			while (num > 0)
			{
				while (pos < m_width)
				{
					m_image[line * m_width + pos] = false;
					++pos;
				}
				--line;
				pos = 0;
				--num;
			}
			break;
		default:
			while (num > 0)
			{
				m_image[line * m_width + pos] = (*p == 'o');
				++pos;
				--num;
			}
		}
		++p;
	}
	while (pos < m_width)
	{
		m_image[line * m_width + pos] = false;
		++pos;
	}
}

void CellMap::addRef()
{
	m_ref++;
}

void CellMap::delRef()
{
	--m_ref;
	if (m_ref <= 0)
	{
		delete this;
	}
}

void CellMap::getSize(long & xMin, long & xMax, long & yMin, long & yMax)
{
	xMin = -m_dx;
	xMax = m_width - m_dx - 1;
	yMin = m_dy - m_height + 1;
	yMax = m_dy;
}
