// ComponentRegister.h: interface for the ComponentRegister class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_COMPONENTREGISTER_H__0DB1B053_A293_43FD_B4B8_A5C1A66BCE9A__INCLUDED_)
#define AFX_COMPONENTREGISTER_H__0DB1B053_A293_43FD_B4B8_A5C1A66BCE9A__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <deque>
#include <queue>
#include <functional>
#include <vector>

#include "CompoundComponent.h"

class AcTree;
class AcTreeNode;
class BasicComponent;
class ComponentVariant;
class CellMap;

class ComponentRegister  
{
public:
	AcTreeNode * getAcTreeRoot();
	bool canFollow(ComponentVariant * var, BasicComponent * com);
	bool canFollow(ComponentVariant * var1, ComponentVariant * var2);
	bool canFollow(BasicComponent * com, ComponentVariant * var);
	bool canFollow(BasicComponent * com1, BasicComponent * com2);
	void prepare(long period, long compression, bool closed, bool useCache);
	void buildTree(long treeSize);

	ComponentRegister(const char * dataFileName);

	virtual ~ComponentRegister();

	BasicComponent * 
	getComponentById(
		long id);

	ComponentVariant * 
	getVariantById(
		long id);

    CompoundComponent * getInvalidSequence();

	inline long getMaxVariants() { return m_maxVariants; };
	inline long getNumComponents() { return m_numComponents; };
	inline long getNumVariants() { return m_numVariants; };
	inline long getFinishedTreeLevel() { return m_finishedTreeLevel; };

	BasicComponent * getComponentByName(const char * name, bool quiet);
private:
	long m_finishedTreeLevel;
	ComponentVariant * getVariantByName(const char * name, bool quiet);
	CellMap * parseCellMap(bool img);
	ComponentVariant * parseVariant(char * buffer, BasicComponent * lastComp);
	bool parseInteger(long minValue, long maxValue, long & value);
	BasicComponent * parseComponent(char * buffer);
	bool buildTreeLevel(CompoundComponent * cc, long l, long maxNodes);
	AcTree * m_acTree;
	long m_maxVariants;

	long m_numComponents;
	BasicComponent ** m_componentList;

	long m_numVariants;
	ComponentVariant ** m_variantList;

	bool ** m_variantMatrix;

	typedef std::deque<BasicComponent *> ComponentQueue;
	typedef ComponentQueue::iterator ComponentIterator;
	ComponentQueue m_componentQueue;

	typedef std::deque<ComponentVariant *> VariantQueue;
	typedef VariantQueue::iterator VariantIterator;
	VariantQueue m_variantQueue;

    struct ccless : public std::less<CompoundComponent *>
	{
		bool operator()(CompoundComponent * x, CompoundComponent * y) const 
		{ 
			return x->getNumComponents() > y->getNumComponents();
		}
    };

	typedef std::priority_queue<CompoundComponent *, std::vector<CompoundComponent *>, ccless> SequenceList;
	SequenceList m_sequenceList;

};

inline 
BasicComponent * 
ComponentRegister::getComponentById(
	long id)
{ 
	return m_componentList[id]; 
};

inline
ComponentVariant * 
ComponentRegister::getVariantById(
	long id) 
{ 
	return m_variantList[id]; 
};

#endif // !defined(AFX_COMPONENTREGISTER_H__0DB1B053_A293_43FD_B4B8_A5C1A66BCE9A__INCLUDED_)
