// CompoundComponent.h: interface for the CompoundComponent class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_COMPOUNDCOMPONENT_H__1B245C7F_C218_4B3B_ABAC_8DCBFB0F2786__INCLUDED_)
#define AFX_COMPOUNDCOMPONENT_H__1B245C7F_C218_4B3B_ABAC_8DCBFB0F2786__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#pragma warning(disable: 4786)
#endif // _MSC_VER > 1000

#include "Component.h"
#include "BasicComponent.h"
#include "ShiftedComponent.h"
#include <deque>
#include "Identifier.h"	// Added by ClassView

class BasicComponent;
class ShiftedComponent;

class CompoundComponent
{
public:
	long checkMinArea(long maxArea);
	inline void setPos(long x, long y, Direction dir)
	{
		m_curX = x;
		m_curY = y;
		m_curDir = dir;
	};
	inline void reset()
	{
		m_curX = 0;
		m_curY = 0;
		m_curDir = TF;
		m_numComponents = 0;
	};

	inline ShiftedComponent * getShiftedComponent(long i) { return &m_list[i]; };
	inline long getComponentId(long i) { return m_list[i].getBasicComponent()->getId(); };
	inline long getNumComponents() { return m_numComponents; };
	void addVariant(long dx, long dy, Direction dir, BasicComponent * comp, ComponentVariant * var);
	void addVariant(BasicComponent * comp, ComponentVariant * var);
	long getArea();
	long getMinArea();
	void getMinSize(long & xMin, long & xMax, long & yMin, long & yMax);
	void printRle(bool forceOrientation);

	inline void addComponent(BasicComponent * comp)
	{
		ShiftedComponent * sc = &m_list[m_numComponents];
		++m_numComponents;
		sc->init(comp, m_curX, m_curY, m_curDir);
		m_curX = sc->getDx();
		m_curY = sc->getDy();
		m_curDir = sc->getDirection();
	};

	inline void removeLastComponent()
	{
		--m_numComponents;
		ShiftedComponent * sc = &m_list[m_numComponents];

		m_curX = sc->getSx();
		m_curY = sc->getSy();
		m_curDir = sc->getSDirection();
	};

	virtual long check(bool optimize);
	virtual CompoundComponent * clone(Direction dir);
	virtual bool getCell(long x, long y);
	void getSize(long &xMin, long &xMax, long &yMin, long &yMax);
	virtual void print(long setNo);
	CompoundComponent(long numComponents, long maxVariants, long matrixSize);
	CompoundComponent(long numComponents, long maxVariants, long matrixSize, char *str);
	virtual ~CompoundComponent();

	inline long getCurX() { return m_curX; };
	inline long getCurY() { return m_curY; };
	inline Direction getCurDir() { return m_curDir; };

private:
	long m_maxVariants;
	long m_maxComponents;
	long m_numComponents;
	ShiftedComponent * m_list;
	long m_area;
	long m_curX;
	long m_curY;
	Direction m_curDir;
	char *m_string;

	struct ComponentRecord;

	struct VariantRecord
	{
		// true if this variant is still to be considered
		bool active; 
		// pointer to the component
		ComponentRecord * component;
		// number of (previous) components blocking this variant
		// zero means that the variant can be used in the search
		long blocking;
		// variant bounding rectangle
		long xMin, xMax, yMin, yMax;
		// variant size
		long size;
		// "physical" variant number (to be used when calling setVariantNo())
		long number;
		// pointer to the matrix row of this variant
		bool * matrixRow;
		// index of this variant in the variantVector
		long index;		
	};

	VariantRecord * m_variantVector;

	struct ComponentRecord
	{
		// number of variants loaded to this component
		long numVariants;
		// number of variants still active
		long activeVariants;
		// number of non-blocked variants
		long freeVariants;
		// pointer to the first variant of this component in the variants vector
		// it may not be active!
		VariantRecord * firstVariant; 
		// pointer one position after the last variant of this component
		VariantRecord * lastVariant;
		// component size
		long size;
		// component number
		long number;
		// active variant (stack record)
		VariantRecord * activeVariant;
		// component reordering pointer
		// for optimization run
		ComponentRecord * reorder;
		// Component size during optimization
		// applies directly to stack level, not to reordered record
		long xMin, xMax, yMin, yMax;
	};

	ComponentRecord * m_componentVector;


	// matrix indexed by variant number. Contains TRUE on offset X*m_matrixSize+Y
	// if the variant X doesn' collide with variant Y
	// X any Y must be from different components
	bool * m_variantMatrix;

	// the matrix width
	long m_matrixSize;
};

#endif // !defined(AFX_COMPOUNDCOMPONENT_H__1B245C7F_C218_4B3B_ABAC_8DCBFB0F2786__INCLUDED_)
