// Writer.cpp: implementation of the Writer class.
//
//////////////////////////////////////////////////////////////////////

#include "Writer.h"
#include <string.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

const char * Writer::s_filename = NULL;
FILE * Writer::s_file = NULL;

const char * Writer::s_rleFilename = NULL;
FILE * Writer::s_rleFile = NULL;

char Writer::s_buffer[80] = { '\0' };

long Writer::s_blankCount = 0;
long Writer::s_nlCount = 0;
long Writer::s_liveCount = 0;

long Writer::s_bufLen = 0;

Writer::Writer()
{

}

Writer::~Writer()
{

}

void Writer::setFilename(const char * filename)
{
	s_filename = filename;
}

void Writer::print(const char *text)
{
	fprintf(stdout, "%s", text);
	if (s_filename != NULL)
	{
		if (s_file == NULL)
		{
			s_file = fopen(s_filename, "w");
			if (s_file == NULL)
			{
				s_filename = NULL;
			}
		}
		if (s_file != NULL)
		{
			fprintf(s_file, "%s", text);
		}
	}
}

void Writer::flush()
{
	if (s_file != NULL)
	{
		fflush(s_file);
	}

	if (s_rleFile != NULL)
	{
		fflush(s_rleFile);
	}
}

void Writer::setRleFilename(const char *filename)
{
	s_rleFilename = filename;
}

void Writer::printRle(const char * text)
{
	if (s_rleFilename != NULL)
	{
		if (s_rleFile == NULL)
		{
			s_rleFile = fopen(s_rleFilename, "w");
			if (s_rleFile == NULL)
			{
				s_rleFilename = NULL;
			}
			else
			{
				fprintf(s_rleFile, "x = 1000, y = 1000, rule = B3/S23\n");
			}
		}
		if (s_rleFile != NULL)
		{
			fprintf(s_rleFile, "%s", text);
		}
	}
}

bool Writer::rleActive()
{
	return s_rleFilename != NULL;
}

void Writer::putRle(char c)
{
	switch (c)
	{
	case 'b':
		if (s_liveCount > 0)
		{
			putRleSeq(s_liveCount, 'o');
			s_liveCount = 0;
		}
		++s_blankCount;
		break;

	case '$':
		if (s_liveCount > 0)
		{
			putRleSeq(s_liveCount, 'o');
			s_liveCount = 0;
		}
		s_blankCount = 0;
		++s_nlCount;
		break;

	default:
		if (s_nlCount > 0)
		{
			putRleSeq(s_nlCount, '$');
			s_nlCount = 0;
		}
		if (s_blankCount > 0)
		{
			putRleSeq(s_blankCount, 'b');
			s_blankCount = 0;
		}
		++s_liveCount;
		break;
	}
}

void Writer::putRleSeq(long len, char c)
{
	char buf[80];

	if (len == 1)
	{
		buf[0] = c;
		buf[1] = '\0';
	}
	else if (len == 2)
	{
		buf[0] = c;
		buf[1] = c;
		buf[2] = '\0';
	}
	else
	{
		sprintf(buf, "%d%c", len, c);
	}
	long bufLen = strlen(buf);

	if (bufLen + s_bufLen + 3 > sizeof(s_buffer))
	{
		strcat(s_buffer, "\n");
		printRle(s_buffer);
		s_buffer[0] = '\0';
		s_bufLen = 0;
	}

	strcat(s_buffer, buf);
	s_bufLen += bufLen;
}

void Writer::flushRle()
{
	s_blankCount = 0;
	if (s_liveCount > 0)
	{
		putRleSeq(s_liveCount, 'o');
		s_liveCount = 0;
	}
	if (s_nlCount > 0)
	{
		putRleSeq(s_nlCount, '$');
		s_nlCount = 0;
	}
	if (s_bufLen > 0)
	{
		strcat(s_buffer, "\n");
		printRle(s_buffer);
		s_buffer[0] = '\0';
		s_bufLen = 0;
	}
}

void Writer::close()
{
	if (s_file != NULL)
	{
		fclose(s_file);
	}

	if (s_rleFile != NULL)
	{
		printRle("!\n");
		fclose(s_rleFile);
	}
}
